#include "Renderer.h"
#include "../Scene/Components.h"
#include "../Graphics/Memory/VertexArray.h"
#include "../Graphics/Memory/Buffer.h"
#include <glad/glad.h>

float Angle = 0.0;
Camera cam = Camera(glm::vec3(12.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.0f), 90.0f);
glm::mat4 projection = glm::perspective(glm::radians(cam.Zoom), (800.0f / 600.0f), 0.001f, 100.0f);

BarinkEngine::Renderer::Renderer(){}

BarinkEngine::Renderer::~Renderer(){}

void BarinkEngine::Renderer::Prepare(Scene& scene ) {
	auto group = scene.getReg().view<BarinkEngine::Render3DComponent>();
	group.each([](auto enity, BarinkEngine::Render3DComponent& renderComponent) {
		VertexArray va = VertexArray();
		GpuBuffer vertexBuffer = GpuBuffer();
		GpuBuffer elementBuffer = GpuBuffer();

		va.Create();
		va.Bind();

		vertexBuffer.createBuffer();
		vertexBuffer.Bind(false);
		vertexBuffer.setBufferData((void*)&renderComponent.mesh.vertices[0], renderComponent.mesh.vertices.size() * sizeof(BarinkEngine::Vertex), false);

		elementBuffer.createBuffer();
		elementBuffer.Bind(true);
		elementBuffer.setBufferData((void*)&renderComponent.mesh.elements[0], renderComponent.mesh.elements.size() * sizeof(unsigned int), true);

		va.AttachAttribute(0, 3, sizeof(BarinkEngine::Vertex));

		glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, sizeof(BarinkEngine::Vertex), (void*)0);
		glEnableVertexAttribArray(0);
		glEnableVertexAttribArray(1);

		va.Unbind();
		vertexBuffer.Unbind(false);
		elementBuffer.Unbind(true);

		renderComponent.VAO = va.getID();
		renderComponent.IBO = elementBuffer.getBufferID();
	});
}

void BarinkEngine::Renderer::Render(Scene& scene)
{
	auto group = scene.getReg().view<TransformComponent, Render3DComponent>();
	group.each([&](auto entity , TransformComponent& trans, Render3DComponent& renderComponent) 
		{
		glBindVertexArray(renderComponent.VAO);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, renderComponent.IBO);

		renderComponent.shader.Use();

		auto lights = scene.getReg().view<LightComponent>();
		lights.each([&](auto entity, LightComponent& light) {
		  	renderComponent.shader.setUniformVec3("lighting.color", light.Color);
			renderComponent.shader.setUniformFloat("lighting.strength", light.Strength);
		});

		renderComponent.shader.setUniformVec3("Color", renderComponent.color);
		renderComponent.shader.setUniformMat4("M", trans.transform);
		renderComponent.shader.setUniformMat4("V", cam.GetViewMatrix());
		renderComponent.shader.setUniformMat4("P", projection);


		glDrawElements(GL_TRIANGLES, static_cast<unsigned int>(renderComponent.mesh.elements.size()) ,
			GL_UNSIGNED_INT, NULL);

		glBindVertexArray(0);
	});

}

void Render(Framebuffer& framebuffer)
{}